package question3;

import question1.PilePleineException;
import question1.PileVideException;

import java.util.Stack;

public class Pile2<T> implements PileI<T>, Cloneable{
    
    /** par delegation : utilisation de la class Stack */
    private Stack<T> stk;

    /** la capacite de la pile */
    private int capacite = 0;

    /** Creation d'une pile.
     * @param taille la taille de la pile, la taille doit etre > 0
     */
    public Pile2(int taille){
        if (taille<=0) this.capacite = CAPACITE_PAR_DEFAUT;
        else this.capacite = taille;
        this.stk = new Stack<T>();
    }

    // constructeur fourni
    public Pile2(){
        this(0);
    }

   /**
   * Ajoute un objet dans la pile.
   */
    public void empiler(T o) throws PilePleineException{
        if (estPleine()) throw new PilePleineException();
        stk.push(o);
    }

   /**
   * Enlve et retourne un objet dans la pile.
   * @return objet de la pile retir
   */
    public T depiler() throws PileVideException{
        if (estVide()) throw new PileVideException();
        return stk.pop();
    }

    /**
     * Retourne le dernier objet de la pile.
     * @return dernier objet
     */
    public T sommet() throws PileVideException{
        if (estVide()) throw new PileVideException();
        return stk.peek();
    }

    /** Effectue un test de l'etat de la pile.
     * @return vrai si la pile est vide, faux autrement
     */
    public boolean estVide(){
        return stk.empty();
    }

    /** Effectue un test de l'etat de la pile.
     * @return vrai si la pile est pleine, faux autrement
     */   
    public boolean estPleine(){
        return stk.size() == this.capacite;
    }

    /** Retourne une representation en String d'une pile, 
     * contenant la representation en String de chaque element.
     * @return une representation en String d'une pile
     */ 
    public String toString(){
        StringBuffer s = new StringBuffer();
        s.append("[");
        for (int i=stk.size()-1;i>=0;i--) {
              s.append(stk.elementAt(i).toString());
              if (i >0) s.append(", ");
            }
        s.append("]");
        return s.toString();
    }

    /**Compare deux piles entre elles.
     * @return vrai     si la pile en entre n'est pas nulle
     *                  si la pile en entre implmente PileI
     *                  si les deux piles ont la meme taille
     *                  si les lments des deux piles correspondent deux a deux
     */
    @Override
    public boolean equals(Object o){
        return  (Pile2<T>)o != null && PileI.class.isInstance(o) && stk.size() == ((Pile2<T>)o).stk.size() && stk.equals(((Pile2<T>)o).stk);
    }

    // fonction fournie
    public int hashCode(){
        return toString().hashCode();
    }

    // fonction fournie
    public Pile2<T> clone() throws CloneNotSupportedException{
        Pile2<T> p = new Pile2<T>(this.capacite());
        p.stk = (Stack<T>)stk.clone();
        return p;
    }

    /** Retourne le nombre d'element d'une pile.
     * @return le nombre d'element present
     */ 
    public int taille(){
        return stk.size();
    }

    /** Retourne la capacite de cette pile.
     * @return le nombre d'element possible
     */ 
    public int capacite(){
        return this.capacite;
    }

} // Pile2.java
